package ru.dbotthepony.mc.otm.capability.fluid

import net.minecraft.nbt.CompoundTag
import net.minecraft.world.item.BlockItem
import net.minecraft.world.item.ItemStack
import net.minecraftforge.common.util.INBTSerializable
import net.minecraftforge.fluids.FluidStack
import ru.dbotthepony.mc.otm.core.nbt.set
import ru.dbotthepony.mc.otm.core.tagNotNull
import ru.dbotthepony.mc.otm.network.synchronizer.IMutableField
import java.util.function.IntSupplier

/**
 * Fluid handler for blocks
 */
open class BlockMatteryFluidHandler(private val _capacity: IntSupplier, field: IMutableField<FluidStack>) : AbstractMatteryFluidHandler(), INBTSerializable<CompoundTag?> {
	override var fluid by field

	override val capacity: Int
		get() = _capacity.asInt

	override fun serializeNBT(): CompoundTag {
		return fluid.writeToNBT(CompoundTag())
	}

	override fun deserializeNBT(nbt: CompoundTag?) {
		fluid = FluidStack.loadFluidStackFromNBT(nbt)
	}

	/**
	 * Fluid handler for items representing block with [BlockMatteryFluidHandler]
	 */
	open class Item(itemStack: ItemStack, capacity: IntSupplier, private val nbtName: String) : ItemMatteryFluidHandler(itemStack, capacity) {
		override var fluid: FluidStack
			get() {
				val sub = itemStack.tag?.get(BlockItem.BLOCK_ENTITY_TAG) as? CompoundTag ?: return FluidStack.EMPTY
				return FluidStack.loadFluidStackFromNBT(sub[nbtName] as? CompoundTag ?: return FluidStack.EMPTY)
			}
			set(value) {
				if (value.isEmpty) {
					val tag = itemStack.tag ?: return
					val subTag = tag.get(BlockItem.BLOCK_ENTITY_TAG) as? CompoundTag

					if (subTag == null) {
						if (tag.isEmpty) {
							itemStack.tag = null
						}
					} else {
						subTag.remove(nbtName)

						if (subTag.isEmpty) {
							tag.remove(BlockItem.BLOCK_ENTITY_TAG)

							if (tag.isEmpty) {
								itemStack.tag = null
							}
						}
					}
				} else {
					var sub = itemStack.tagNotNull.get(BlockItem.BLOCK_ENTITY_TAG) as? CompoundTag

					if (sub == null) {
						sub = CompoundTag()
						itemStack.tagNotNull[BlockItem.BLOCK_ENTITY_TAG] = sub
					}

					sub[nbtName] = value.writeToNBT(CompoundTag())
				}
			}
	}
}
