package ru.dbotthepony.mc.otm.core

import net.minecraft.ChatFormatting
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.Tag
import net.minecraft.network.chat.Component
import net.minecraft.world.item.BlockItem
import net.minecraft.world.item.DyeColor
import net.minecraft.world.item.ItemStack
import net.minecraftforge.common.capabilities.ForgeCapabilities
import ru.dbotthepony.mc.otm.block.entity.MatteryBlockEntity
import ru.dbotthepony.mc.otm.capability.energy
import ru.dbotthepony.mc.otm.capability.energy.BlockEnergyStorageImpl
import ru.dbotthepony.mc.otm.capability.energy.CapacitorEnergyStorage
import ru.dbotthepony.mc.otm.capability.energy.GeneratorEnergyStorage
import ru.dbotthepony.mc.otm.capability.energy.IMatteryEnergyStorage
import ru.dbotthepony.mc.otm.capability.energy.ItemEnergyStorageImpl
import ru.dbotthepony.mc.otm.capability.energy.WorkerEnergyStorage
import ru.dbotthepony.mc.otm.capability.energy.batteryLevel
import ru.dbotthepony.mc.otm.capability.fluidLevel
import ru.dbotthepony.mc.otm.capability.matter.MatterStorageImpl
import ru.dbotthepony.mc.otm.client.ShiftPressedCond
import ru.dbotthepony.mc.otm.client.isShiftDown
import ru.dbotthepony.mc.otm.client.minecraft
import ru.dbotthepony.mc.otm.config.ClientConfig
import ru.dbotthepony.mc.otm.container.MatteryContainer
import ru.dbotthepony.mc.otm.core.math.Decimal
import ru.dbotthepony.mc.otm.core.nbt.map
import ru.dbotthepony.mc.otm.core.nbt.mapPresent
import ru.dbotthepony.mc.otm.core.util.formatMatter
import ru.dbotthepony.mc.otm.core.util.formatPower

class TooltipList {
	private val descriptionLines = ArrayList<(itemStack: ItemStack, acceptor: (text: Component) -> Unit) -> Unit>()

	fun assemble(itemStack: ItemStack, into: (Component) -> Unit) {
		if (descriptionLines.isNotEmpty()) {
			if (ClientConfig.HIDE_DESCRIPTION && !minecraft.window.isShiftDown && descriptionLines.any { var flag = false; it.invoke(itemStack) { flag = true }; flag }) {
				into.invoke(TranslatableComponent("otm.gui.shift_for_more_info").withStyle(ChatFormatting.GRAY).withStyle(ChatFormatting.ITALIC))
			} else {
				for (lines in descriptionLines) {
					lines.invoke(itemStack, into)
				}
			}
		}
	}

	fun assemble(itemStack: ItemStack, into: MutableCollection<Component>) {
		assemble(itemStack, into::add)
	}

	fun addNormal(function: (itemStack: ItemStack, acceptor: (text: Component) -> Unit) -> Unit) {
		descriptionLines.add(function)
	}

	@JvmName("addPlain")
	fun add(component: Component) {
		descriptionLines.add { _, result -> result.invoke(component.copy()) }
	}

	@JvmName("addFunction2Component")
	fun add(component: () -> Component) {
		descriptionLines.add { _, result -> result.invoke(component.invoke()) }
	}

	fun needsNoPower(formatting: ChatFormatting = ChatFormatting.GRAY) {
		add(TranslatableComponent("otm.needs_no_power").withStyle(formatting))
	}

	fun painted(color: DyeColor?) {
		if (color != null) {
			add(TranslatableComponent("otm.painted.${color.name.lowercase()}").withStyle(ChatFormatting.DARK_GRAY))
		}
	}

	fun colored(color: DyeColor?) = painted(color)

	inline fun <reified T : Tag> blockEntityData(key: String, noinline callback: (itemStack: ItemStack, data: T, acceptor: (line: Component) -> Unit) -> Unit) {
		addNormal { itemStack, acceptor ->
			val tag = (itemStack.tag?.get(BlockItem.BLOCK_ENTITY_TAG) as? CompoundTag)?.get(key) as? T ?: return@addNormal
			callback(itemStack, tag, acceptor)
		}
	}

	fun blockEntityEnergy(energyKey: String = MatteryBlockEntity.ENERGY_KEY, batteryKey: String = MatteryBlockEntity.BATTERY_KEY) {
		blockEntityData<CompoundTag>(energyKey) { _, tag, acceptor ->
			val stored = tag.mapPresent(BlockEnergyStorageImpl.ENERGY_STORED_KEY, Decimal::deserializeNBT)

			if (stored != null) {
				acceptor(
					TranslatableComponent(
						"otm.item.power.storage0",
						stored.formatPower(formatAsReadable = ShiftPressedCond)
					).withStyle(ChatFormatting.GRAY))
			}

			val container = MatteryContainer(1)
			tag.map(batteryKey, container::deserializeNBT)

			if (!container[0].isEmpty) {
				acceptor(TranslatableComponent("otm.item.block.stored_battery", container[0].displayName).withStyle(ChatFormatting.GRAY))

				val energy = container[0].energy ?: return@blockEntityData

				if (energy is IMatteryEnergyStorage) {
					batteryLevel(energy, acceptor)
				} else {
					batteryLevel(energy, acceptor)
				}
			}
		}
	}

	fun blockEntityMatter(matterKey: String = MatteryBlockEntity.MATTER_STORAGE_KEY) {
		blockEntityData<CompoundTag>(matterKey) { _, tag, acceptor ->
			val stored = tag.mapPresent(MatterStorageImpl.MATTER_STORED_KEY, Decimal::deserializeNBT)

			if (stored != null) {
				acceptor(
					TranslatableComponent(
						"otm.item.matter.storage0",
						stored.formatMatter(formatAsReadable = ShiftPressedCond)
					).withStyle(ChatFormatting.GRAY))
			}
		}
	}

	fun itemEnergy() {
		addNormal { itemStack, acceptor ->
			val energy = itemStack.energy ?: return@addNormal

			if (energy is IMatteryEnergyStorage) {
				batteryLevel(energy, acceptor)
			} else {
				batteryLevel(energy, acceptor)
			}
		}
	}

	fun doubleProcessing() {
		add(TranslatableComponent("otm.gui.double_processing").withStyle(ChatFormatting.GRAY))
	}

	fun itemFluid() {
		addNormal { itemStack, acceptor ->
			itemStack.getCapability(ForgeCapabilities.FLUID_HANDLER_ITEM).ifPresentK {
				it.fluidLevel(acceptor)
			}
		}
	}
}
